extensions [ gis ]
globals 
 [divisions
  fractal-dim 
  initial-length
  len 
  walk-count  
  old-box-count 
  new-box-count   
  x-axis-list
  y-axis-list
  box-size 
  explore?
  iteration
  iterations-to-go
  slope
  lin-reg-eq
  elevation slope2 aspect
  ]
turtles-own
  [new?]
breed 
 [boxes box]
boxes-own 
  [current-step
   next-step]

to startup
  setup
end


to setup
  clear-all
  
  ;;;;;;;;;;;;;;;;;;;;;;;;
  ;;;;;;import image;;;;;;
  ;;;;;;;;;;;;;;;;;;;;;;;;
  
  if coastline?
  [
import-pcolors  "coast-of-gb.png"
ask patches with [pcolor != white]
[set pcolor black]
  ]
  if biology?
  [
 import-pcolors "LepMEpSelfOrganization.png"
  ]
  if face?
  [
import-pcolors "Mandelbrot.png"
  ]
if your-image?
  [
import-pcolors    "Chanson-concentric-cell.png"
  ]

 set explore? false
 set x-axis-list [ ]
 set y-axis-list [ ]
 ask turtles [die]
 ask boxes [die]  
 set initial-length 20
  reset-ticks
end




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;Box-Counting-Dim;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to box-counting-dimension-setup
   ask boxes ; clears screen in case there are any old boxes
   [die]
  set box-size initial-length * initial-scaling-factor
  set old-box-count 0
  set new-box-count 1
  create-boxes 1
  ask boxes [
    set shape "square"
    set size box-size
    setxy -55 -64
    set heading 90
    fd box-size / 4
    set color box-color
    set current-step 1
    set next-step current-step
    set iteration 0
    set explore? false
    ]
end

to box-counting-dimension
    ask boxes with [color = box-color and next-step = current-step ] [
    ifelse old-box-count < new-box-count
    [ 
      hatch 1 fd box-size rt 90 
      set next-step current-step + 1
      hatch 1 fd box-size rt 90 
      set next-step current-step + 1
      hatch 1 fd box-size
      set next-step current-step + 1
      hatch 1 fd box-size rt 90
      set next-step current-step + 1
      hatch 1 fd box-size 
      set next-step current-step + 1 
      hatch 1 fd box-size rt 90
      set next-step current-step + 1
      hatch 1 fd box-size
      set next-step current-step + 1 
      hatch 1 fd box-size 
      set next-step current-step + 1
       ]
    [stop]
    ]
      ask boxes with [color = box-color and next-step = current-step + 1] [
      if old-box-count < new-box-count
      [cover]
      ]
      ask boxes [
       set current-step current-step + 1
      ]
     set old-box-count new-box-count
     set new-box-count count boxes with [color = box-color]
     if old-box-count = new-box-count
      [ explore ]
      ifelse old-box-count = new-box-count and  automatic-box-count? [
       automatic-box-count-go ]
      [if old-box-count = new-box-count
      [stop]
     ]
 tick
end
      
to cover
   if count boxes  in-radius (1.4 * box-size / 2 )  > 1  ; eliminates duplicates
   [die]
   set color black 
   ifelse count patches in-radius  ( (1.4 * box-size ) / 2 )  with [pcolor = color-value] > 0         
   [set color box-color ]
   [ die ]
end

to explore
 ask patches with [pcolor = color-value] [
 ifelse count boxes in-radius  (1.4 * box-size ) = 0 [set explore? true]
  [stop]
 ]
 
 if explore? [
 ask one-of boxes with [next-step = current-step - 1] [
 hatch 1
 set current-step current-step + 1
 move-to min-one-of patches with [pcolor = color-value and count boxes in-radius  (1.4 *  box-size  ) = 0 ]  
 [distance myself]
 set next-step current-step
 set new-box-count count boxes with [color = box-color]
 set explore? false
 stop
   ]
 ]

 ifelse new-box-count > old-box-count
 [ box-counting-dimension]
 [calculate-box-counting-dimension
 stop]
end

to automatic-box-count-go
    if automatic-box-count?
    [
    ifelse box-size <= initial-length 
      [ask boxes [die]
     set box-size box-size + increment 
     set old-box-count 0
     set new-box-count 1
     create-boxes 1
     ask boxes [
     set shape "square"
     set size box-size
     setxy -55 -64
     set heading 90
     fd box-size / 4
     set color box-color
     set next-step current-step
     set iteration iteration + 1
     set iterations-to-go (((initial-length / increment) - (initial-length * initial-scaling-factor)) - iteration + 1)
     ]
     box-counting-dimension ]
    [ set automatic-box-count? false] ]
end
      
to calculate-box-counting-dimension
 if plot-box-count?  [
 if count boxes >= 1 [  ; eliminates potential error message if setup is pressed during a box-counting procedure
 set-current-plot "scatter-plot"
 set-current-plot-pen "default"
 let no-boxes log (count boxes ) 10
 let scale (log ( 1 / box-size ) 10 )
 plotxy scale no-boxes
 set x-axis-list lput scale x-axis-list
 set y-axis-list lput no-boxes y-axis-list
   ]
  ] 
 stop 

end




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;Linear Reg;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to linear-regression

  if count boxes >= 1 [  ; eliminates potential error message if setup is pressed during a box-counting procedure [
  let sum-x sum x-axis-list
  let sum-y sum y-axis-list
  let # length x-axis-list
  
  let ave-x sum-x / #
  let ave-y sum-y / #
  let sum-xy sum ( map  [ ?1 * ?2 ] x-axis-list y-axis-list ) 

  let sum-x-square sum  ( map  [ ? * ? ] x-axis-list ) 

  let sum-y-square sum  ( map [ ? * ? ] y-axis-list )

  ; compute and store the denominator for the slope and y-intercept 
  let denominator ( sum-x-square - # * ( ave-x ^ 2 ) )

  ; if the denominator = 0 or the min of xcors = the max of xcors, the turtles are in a vertical line
  ; thus, the line of regression will be undefined and have a r^2 value of 1.0
  ifelse ( denominator = 0 ) 
  [
    set slope "Undefined"
    let y-intercept "Undefined"
    
  ]
  [
   ; otherwise, there is some other type of line.  so find the y-intercept and slope of the line

    let y-intercept ( ( ave-y * sum-x-square ) - ( ave-x * sum-xy ) ) / denominator
    set slope precision (( sum-xy - # * ave-x * ave-y ) / denominator)3

    ;; compute the value of r^2
    ifelse ( ( sum-y-square - # * ( ave-y ^ 2 ) ) = 0 ) 
    [ let r-square 1 ]
    [
      let r-square precision (( ( sum-xy - # * ave-x * ave-y ) ^ 2 ) /
        ( ( sum-x-square - # * ( ave-x ^ 2 ) ) * ( sum-y-square - # * ( ave-y ^ 2 ) ) )) 3
        
    ]
    

 ; set the equation to the appropriate string
 set lin-reg-eq (word (precision slope 3) " * x + " (precision y-intercept 3))

 set-current-plot "scatter-plot"
 set-current-plot-pen "pen-4"
 plotxy .2 .2 * slope + y-intercept
 plotxy -2 -2 * slope + y-intercept
    ]
   ]
end
@#$#@#$#@
GRAPHICS-WINDOW
311
11
842
563
260
260
1.0
1
10
1
1
1
0
1
1
1
-260
260
-260
260
1
1
1
ticks
30.0

BUTTON
15
55
141
88
NIL
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SLIDER
15
223
172
256
initial-scaling-factor
initial-scaling-factor
.02
1
0.29
.01
1
NIL
HORIZONTAL

BUTTON
15
255
263
288
box-counting-dimension-setup
box-counting-dimension-setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

PLOT
851
10
1118
265
scatter-plot
log [1 / Scale factor (r)]
log number of boxes (N)
-2.0
-1.0
0.0
2.0
true
false
"" ""
PENS
"default" 1.0 2 -16777216 true "" ""
"pen-4" 1.0 0 -2674135 true "" ""

MONITOR
916
418
981
463
no-boxes
count boxes with [color = red]
17
1
11

BUTTON
15
288
263
321
box-counting-dimension-go
box-counting-dimension
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

MONITOR
851
418
916
463
patches
count patches with [pcolor = green]
17
1
11

SWITCH
15
359
187
392
plot-box-count?
plot-box-count?
0
1
-1000

SWITCH
15
391
187
424
automatic-box-count?
automatic-box-count?
0
1
-1000

TEXTBOX
16
427
199
497
Switch plot-box-count? On to plot  box-counting dimension. Switch automatic-box-count? On to do a series of runs.
11
0.0
1

BUTTON
13
550
185
583
NIL
linear-regression
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

MONITOR
851
265
965
310
box-counting-dim
slope
17
1
11

SLIDER
14
486
186
519
increment
increment
.1
1
0.1
.1
1
NIL
HORIZONTAL

TEXTBOX
16
521
216
545
Increment sets amount box-size increases for each iteration
11
0.0
1

TEXTBOX
15
589
206
628
Linear regression determines the\n'best fit' line for the scatter-plot
11
0.0
1

MONITOR
980
418
1045
463
NIL
box-size
17
1
11

MONITOR
852
462
917
507
NIL
iteration
17
1
11

MONITOR
916
462
1045
507
NIL
iterations-to-go
17
1
11

MONITOR
965
265
1118
310
NIL
lin-reg-eq
17
1
11

TEXTBOX
855
514
1104
570
monitors report variables during\neach iteration of automatic-box-count
11
0.0
1

TEXTBOX
855
316
1087
349
box-counting-dimension is the slope of the linear regression
11
0.0
1

SLIDER
140
55
274
88
color-value
color-value
0
139
0
1
1
NIL
HORIZONTAL

SLIDER
172
223
264
256
box-color
box-color
0
139
12
1
1
NIL
HORIZONTAL

SWITCH
15
87
141
120
Coastline?
Coastline?
0
1
-1000

SWITCH
141
87
274
120
biology?
biology?
1
1
-1000

SWITCH
15
119
140
152
face?
face?
1
1
-1000

SWITCH
140
119
274
152
your-image?
your-image?
1
1
-1000

TEXTBOX
25
156
275
212
Use color swatches under tools and the color-scale slider to measure a patch color
11
0.0
1

TEXTBOX
7
19
459
41
Box-Counting-Dimension Applied
18
0.0
1

TEXTBOX
19
195
259
213
Box-Counting Controls
18
0.0
1

TEXTBOX
18
332
168
354
Plot Controls
18
0.0
1

@#$#@#$#@


## WHAT IS IT?
Box-counting Dimension Applied (intermediate-bcd-applied.nlogo) is a model that illustrates how any 2-D image can be imported into NetLogo as a .png file and its fractal dimension measured. Images representing geographical information systems data (GIS) as well as biological, geometrical, photographic or literally any image are analyzed using fractal dimension and insights drawn regarding the multi-scale detail of such images as well as complex edge conditions, fragmentation etc. Fractal dimension is a tool for understanding and measuring complex forms.

## HOW IT WORKS

Fractal dimension is based on Hausdorff dimension developed by the German mathematician Felix Hausdorff. A line segment or initiator with divisions r in one dimension, D = 1, has a partitioning of N where N = r.  In two dimensions the number of partitions increases to N 2 or 4, this second order operation is termed the generator.  Likewise in three dimensions, D = 3, the number of partitions increases to N 3 or 8.  This relationship is generalized to any geometric condition where N = r D.   

## HOW TO USE IT
In the upper left hand corner of the interface, turn Coastline? On and the rest of the examples off. An image of the coast of Britain should appear in the view field. Use color swatches under tools in the NetLogo menu bar and the color-scale slider in the interface to choose a color gradient. This is what will be measured. In the coastline example the color gradient is black or 0. Next, set the scaling factor to the desired level — say .3 for this example — press box-counting-dimension-setup and then box-counting-dimension-go. A box-counting run should begin. Boxes at the chosen scale cover the chosen color and plot the log of the number of boxes, N, over the log of the scale, 1/r, in the scatter-plot in the upper right hand corner of the interface. If automatic-box-count? is On, multiple runs will automatically plot points until reaching the maximum box size specified in the code under setup. Increment sets amount box-size increases for each iteration. After a sufficient number of runs, press linear-regression in the bottom left hand corner of the interface to determine the box-counting-dimension of the image. It will appear in the monitor box-counting-dimension with the linear regression graphed in the scatter-plot.

## THINGS TO NOTICE
The box-counting dimension is more accurate (equivalent to the fractal dimension) if the initial fractal is iterated more. However, you have probably noticed the fractal taking longer and longer to generate at each successive iteration?  This is because the number of line segments is increasing exponentially.  The Koch curve, for instance, begins with 4 line segments but has over a million line segments after ten iterations.

## Uploading Images

In NetLogo, files using imported images must be in the NetLogo folder. Be sure to store the model and any images you will be using in the NetLogo folder containing the correct version (version you are using) of NetLogo. 

Store the model intermediate-bcd-applied.nlogo in the following folder thread:

NetLogo 5.0 -> models -> code examples -> GIS -> intermediate-fractal-d-applied.nlogo Any images used in the model must be stored here as well and be in the .png file format. Here are the additional files that must be stored in the NetLogo folder for the model to be functional.

coast-of-gb.png
CPM.png
face B&W.png

To upload an image, first save it as a png at a managable size — about 600 X 600 and less than 1 MB. Images can be larger but will take more time to setup. Under Setup you will see if your-image. currently there is the CPM.png file here. Delete that and put your file here in brackets. Here is the correct code for the coastline? procedure.
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -7500403 true true 15 45 15 60
Rectangle -7500403 true true 0 0 300 300

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 5.0RC4
@#$#@#$#@
ballSetup
repeat 14 [ go ]
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
0
@#$#@#$#@
